package LATMOS::Accounts::SyncManager;

use strict;
use warnings;
use LATMOS::Accounts::Log;
use LATMOS::Accounts;

sub new {
    my ($class, $config) = @_;

    my $ini = Config::IniFiles->new(
        -file => join('/', ($config || '/etc/latmos-accounts'),
            'la-sync-manager.ini'),
    ) or do {
        la_log LA_ERR, "Cannot load config";
        return;
    };

    bless {
        ini => $ini,
        config => $config,
        lastrev => 0,
    }, $class;
}

sub ini { $_[0]->{ini} }

sub dbrev { $_[0]->{lastrev} }

sub updrev {
    my ($self) = @_;
    my $LA = LATMOS::Accounts->new($self->{config}, noacl => 1);
    my $base = $LA->default_base;
    my $newrev = $base->current_rev;
    $self->{lastrev} = $newrev;
    return $newrev;
}

sub list_modules {
    my ($self) = @_;
    $self->ini->Sections;
}

sub process_module {
    my ($self, $module, $dbrev) = @_;

    if (!$self->ini->SectionExists($module)) {
        la_log LA_ERR, "Cannot run inexistant module %s", $module;
        return;
    }

    my $modtype = $self->ini->val($module, 'type', $module);
    my $task = LATMOS::Accounts::Task->new(
        $modtype,
        config => $self->{config},
    )
        or do {
        la_log(LA_ERR, 'Cannot load module %s', $modtype);
        return;
    };
    la_log LA_NOTICE, "Processing sync module %s (%s)", $module, $modtype;
    if (!$task->init) {
        la_log(LA_ERR, 'init() failed for module %s', $module);
        return;
    }

    if (!$task->needupd($dbrev, $self)) {
        la_log LA_DEBUG, "No change on main base, aborting";
        return 1;
    }


    my $res = $task->run;
    if (!$res) {
        la_log LA_ERR, "Task %s did not end successfully", $module;
    }
    $task->post if ($res);
    la_log LA_DEBUG, "end process $module";
    return $res;
}

1;
